# dispatch.tcl -- Websh style dispatch

# $Id: dispatch.tcl,v 1.2 2004/04/10 17:45:32 davidw Exp $

package provide dispatch 0.1

package require http

namespace eval dispatch {
    array set state {}
}

proc dispatch::state {name code} {
    variable state
    set state($name) $code
    set page {}
}

# dispatch::statevars --
#
#	Take a list of key/val lists in the form {{foo bar} {bee bop}
#	{shoo wah}} and check to make sure that the keys exist as CGI
#	variables, and create them in the local namespace as regular
#	Tcl variables.
#
# Arguments:
#	varlist.
#
# Side Effects:
#	Create local Tcl variables.
#
# Results:
#	None.

proc dispatch::init {} {
    variable page
    set page [lindex [split [env REQUEST_URI] ?] 0]
}

proc dispatch::statevars {varlist} {
    foreach {key val} $varlist {
	upvar $key localvar
	set localvar [var get $key $val]
    }
}

proc dispatch::cmd {action args} {
    variable page
    set argstr [eval http::formatQuery dispatch_action $action $args]
    return "[makeurl $page?$argstr]"
}

proc dispatch::cmdformurl {} {
    variable page
    puts -nonewline [makeurl $page]
}

proc dispatch::cmdform {action args} {
    puts  "<input type=\"hidden\" name=\"dispatch_action\" value=\"$action\">"
    foreach {key val} $args {
	puts "<input type=\"hidden\" name=\"$key\" value=\"$val\">"
    }
}

proc dispatch::cmdurl {action args} {
    puts -nonewline [eval dispatch::cmd $action $args]
}

proc dispatch::dispatch {} {
    variable state
    # This is where evaluation will happen - make sure it's clean.
    catch { namespace delete Eval }
    namespace eval Eval {}
    set page [info script]

    set action [var get dispatch_action default]
    if { [catch { namespace eval Eval $state($action) } err] } {
	puts "<p>Error carrying out '$action' :</p>
              <hr>
              <pre>$err</pre>
              <hr>"
	if { [info exists errorInfo] } {
	    puts "$errorInfo<hr>"
	}
    }
}
